#!/usr/bin/perl
#
# artist.perl
#
# builds index of all artists, linked with index of songs,
#  sorted and grouped by artist then album.
#
# imports:
#    environment:
#    TUNEBOT_ROOT    root of tunebot installation.
#    TUNETAGS        path of tags database file,
#                       lines in format /.../alb[/art[/trk]]:TAG=value
#    TUNEIX          path of tracks index file,
#                       lines in format /.../alb/art/trk.mp3
#    INDEX_MAX_PER   max number of tracks to coalesce into a single
#                       index page.
#    BANNER_STRING   identifying string for web page titles
#
#    files:
#       $TUNEBOT_ROOT/html/bodypart.html.head
#       $TUNEBOT_ROOT/html/bodypart.html.tail
#
# generates:
#    files:
#       $TUNEBOT_ROOT/html/tunesbyartist??.html  partial indices
#       $TUNEBOT_ROOT/html/artists.html          list of all artists
#       $TUNEBOT_ROOT/tune.autotags              indexing related tags
#

#
# get the bucket tag for a string
#
sub btag {
   my ($a) = @_;
   $a = substr($a, 0, 1);
   if ($a ge "A" && $a le "Z") {
	return $a;
   } elsif ($a ge "a" && $a le "z") {
	return uc($a);
   } else {
	return "A";
   }
}

#
# suck the tags database into a hash indexed by {file, tag}
#
open(TUNETAGS, $ENV{TUNETAGS});
while ($line = <TUNETAGS>) {
    chop $line;
    $i = index($line, ":");
    $j = index($line, "=", $i);
    $file = substr($line, 0, $i);
    $tag = substr($line, $i + 1, $j - $i - 1);
    $value = substr($line, $j + 1);
    $tags{$file, $tag} = $value;
}
close(TUNETAGS);

#
# we will write indexing-related tags in this file, such as:
# - in what by-artist hash bucket to find an artist and album
# - the html NAME tag to skip to the entry
#
open(AUTOTAGS, ">" . $ENV{TUNEBOT_ROOT} . "/tune.autotags");

#
# count all tracks in tune index into alpha buckets by artist,
# using the artist SORT tag, else N tag, else dirname.
# also make hash of all artists, keyed on artist sort-key.
#
%bcount = (
    "A", 0, "B", 0, "C", 0, "D", 0, "E", 0, "F", 0, "G", 0, "H", 0,
    "I", 0, "J", 0, "K", 0, "L", 0, "M", 0, "N", 0, "O", 0, "P", 0,
    "Q", 0, "R", 0, "S", 0, "T", 0, "U", 0, "V", 0, "W", 0, "X", 0,
    "Y", 0, "Z", 0);
$ntunes = 0;
open(TUNEIX, $ENV{TUNEIX});
while ($path = <TUNEIX>) {
    $ntunes++;
    chop $path;
    $i = index($path, "\.mp3");
    if ($i >= 0) {
	$path = substr($path, 0, $i);
    }
    @raw = split('/', $path);
    $n = $#raw;
    $artistfile = join("/", @raw[0..$n-2]);
    $artist = $tags{$artistfile, "N"};
    if ($artist eq "") {
	$artist = $raw[$n-2];
    }
    $sartist = $tags{$artistfile, "SORT"};
    if ($sartist eq "") {
	$sartist = $artist;
    }
    $bcount{btag($sartist)}++;
    $artists{$sartist} = $artist;
    if ($tags{$artistfile, "BOLD"} != 0) {
	$boldartists{$sartist} = 1;
    }
}
close(TUNEIX);

#
# coalesce buckets, up to the limit on number of entries per bucket
#
$indmax = $ENV{INDEX_MAX_PER};
#print "indmax=", $indmax, "\n";
$coal = 1;
while ($coal) {
    @keys = sort(keys(bcount));
    $bmin = -1;
    $bminc = $indmax + 1;
    for ($b = 0; $b < $#keys; $b++) {
#	print $b, " ", $keys[$b], " ", $bcount{$keys[$b]}, "\n";
	if ($bcount{$keys[$b]} + $bcount{$keys[$b + 1]} < $bminc) {
	    $bmin = $b;
	    $bminc = $bcount{$keys[$b]} + $bcount{$keys[$b + 1]};
	}
    }
#    print "bmin=", $bmin, "\n";
    $coal = 0;
    if ($bmin != -1) {
	$bcount{$keys[$bmin] . $keys[$bmin + 1]}
	    = $bcount{$keys[$bmin]} + $bcount{$keys[$bmin + 1]};
	delete $bcount{$keys[$bmin]};
	delete $bcount{$keys[$bmin + 1]};
	$coal = 1;
    }
#    print "--------------------\n";
}

#
# generate list of coalesced buckets: (@cbs)
# and bucket tag to coalesced bucket map: $coalbucket{tag}
#
@cbs = ();
@keys = sort(keys(bcount));
for ($b = 0; $b <= $#keys; $b++) {
#    print $b, " ", $keys[$b], " ", $bcount{$keys[$b]}, "\n";
    $c1 = substr($keys[$b], 0, 1);
    $c2 = substr($keys[$b], length($keys[$b]) - 1);
    @cbs = (@cbs, $c1 . $c2);
    foreach $c (split //, $keys[$b]) {
	$coalbucket{$c} = $c1 . $c2;
#	print "coalbucket of ", $c, " is ", $coalbucket{$c}, "\n";
    }
#    print "\n";
}

#
# for sorting upper and lowercase together
#
sub folded { uc($a) cmp uc($b); }

#
# create all the coalesced-bucket index files.
#  read through track list, picking tracks that go into an index,
#  sort tracks into artist/album
#  generate html
#
#foreach $cb (@cbs) {
for ($cbindex = 0; $cbindex <= $#cbs; $cbindex++) {
    $cb = $cbs[$cbindex];
    $prevcb = "";
    if ($cbindex > 0) {
	$prevcb = $cbs[$cbindex - 1];
    }
    $nextcb = "";
    if ($cbindex < $#cbs) {
	$nextcb = $cbs[$cbindex + 1];
    }
    print " ", "tunesbyartist" . $cb . ".html", "\n";
    $html{$cb} = "HTML" . $cb;
    open($html{$cb}, ">" . $ENV{TUNEBOT_ROOT} . "/html/tunesbyartist" . $cb . ".html");
    print {$html{$cb}} "<html>\n";
    print {$html{$cb}} "<head>\n";
    print {$html{$cb}} "<META HTTP-EQUIV=\"Pragma\" CONTENT=\"no-cache\">\n";
    $c1 = substr($cb, 0, 1);
    $c2 = substr($cb, length($cb) - 1);
    print {$html{$cb}} "<title>" . $ENV{BANNER_STRING} . " Tunes by Artist ", $c1, "-", $c2, "</title>\n";
    print {$html{$cb}} "</head>\n";
    open(I, $ENV{TUNEBOT_ROOT} . "/html/bodypart.html.head");
    while ($line = <I>) {
	print {$html{$cb}} $line;
    }
    close(I);
    print {$html{$cb}} "<center><font size=+2>Tunes by Artist ", $c1, "-", $c2, "</font>\n";
    print {$html{$cb}} "<br>click to make a request</center>\n";
    print {$html{$cb}} "<p><center>\n";
if (0) {
    print {$html{$cb}} "<table width=33%><tr>\n";
    print {$html{$cb}} "<td align=center>\n";
    if ($prevcb ne "") {
	print {$html{$cb}} "<a href=\"tunesbyartist", $prevcb, ".html\"><img src=\"/rfm/upup.gif\" alt=\"browse Back\" border=0></a>\n";
    }
    print {$html{$cb}} "</td>\n";
    print {$html{$cb}} "<td align=center><a href=\"#ENDOFILE\"><img src=\"/rfm/downto.gif\" alt=\"to Bottom\" border=0></a></td>\n";
    print {$html{$cb}} "<td align=center>\n";
    if ($nextcb ne "") {
	print {$html{$cb}} "<a href=\"tunesbyartist", $nextcb, ".html\"><img src=\"/rfm/downdown.gif\" alt=\"browse Fwd\" border=0></a>\n";
    }
    print {$html{$cb}} "</td>\n";
    print {$html{$cb}} "</tr></table>\n";
}
    print {$html{$cb}} "<br><b>\n";
    $i = "";
    foreach $c (sort(keys(coalbucket))) {
	print {$html{$cb}} $i, "<a href=\"tunesbyartist", $coalbucket{$c}, ".html#", $c, "\">", $c, "</a>\n";
	$i = "-";
    }
    print {$html{$cb}} "</b></center><p>\n";

    @sortks = ();
    open(TUNEIX, $ENV{TUNEIX});
    $i = 0;
    while ($path = <TUNEIX>) {
	chop $path;
	$canon = $path;
#	$i = index($path, "\.mp3");
#	if ($i >= 0) {
#	    $path = substr($path, 0, $i);
#	}
	$path =~ s/\.mp3$//;
	@raw = split('/', $path);
	$n = $#raw;
	$artistfile = join("/", @raw[0..$n-2]);
	$artist = $tags{$artistfile, "N"};
	if ($artist eq "") {
	    $artist = $raw[$n-2];
	}
	$sartist = $tags{$artistfile, "SORT"};
	if ($sartist eq "") {
	    $sartist = $artist;
	}
	if ($coalbucket{btag($sartist)} eq $cb) {
	    $albumfile = join("/", @raw[0..$n-1]);
	    $album = $tags{$albumfile, "N"};
	    if ($album eq "") {
		$album = $raw[$n-1];
	    }
	    $date = substr($tags{$albumfile, "DATE"}, 0, 4);
	    if ($date eq "") {
		$date = "0000"
	    }
	    $track = $tags{$path, "N"};
	    if ($track eq "") {
		$track = $raw[$n];
	    }
	    $sortk = $sartist . "/" . $date . "/" . $album . "/" . $raw[$n];
#	    print $canon, "\n";
#	    print " SORTS ON ", $sortk, "\n";
#	    print "  RENDERS AS ", $artist . "/" . $album . "/" . $track, "\n";
#	    print "   GOES INTO ", $coalbucket{btag($sartist)}, "\n";
#	    @sortks = (@sortks, $sortk);
	    $sortks[$i++] = $sortk;
	    $canons{$sortk} = $canon;
#	    $render{$sortk} = join("/", $artist, $album, $track);
	}
    }
    close(TUNEIX);

    @sortks = sort folded @sortks;

    $indent = 0;
    $lastar = "";
    $lastal = "";
    $lastalf = substr($cb, 0, 1);
    $lastalf = chr(ord($lastalf) - 1);
    foreach $sortk (@sortks) {
#	print $sortk, "\n";
#	print " ", $canons{$sortk}, "\n";
#	print "  ";
#	print $render{$sortk};
#	print "\n";
	$canon = $canons{$sortk};
	$path = $canon;
	$i = index($path, "\.mp3");
	if ($i >= 0) {
	    $path = substr($path, 0, $i);
	}
	@raw = split('/', $path);
	$n = $#raw;
	$artistfile = join("/", @raw[0..$n-2]);
	$artist = $tags{$artistfile, "N"};
	if ($artist eq "") {
	    $artist = $raw[$n-2];
	}
	$albumfile = join("/", @raw[0..$n-1]);
	$album = $tags{$albumfile, "N"};
	if ($album eq "") {
	    $album = $raw[$n-1];
	}
	$track = $tags{$path, "N"};
	if ($track eq "") {
	    $track = $raw[$n];
	}
	if ($tags{$path, "ARTIST"} ne "") {
	    $track = $track . "<em> - " . $tags{$path, "ARTIST"} . "</em>";
	}
#	@aat = split('/', $render{$sortk});
	if ($taggedart{$artistfile} == 0) {
	    $taggedart{$artistfile} = 1;
	    print AUTOTAGS $artistfile . ":H_ARTIST=" . $cb . "\n";
	    print AUTOTAGS $artistfile . ":A_ARTIST=" . $artist . "\n";
	}
	$i = 2;
	if ($artist ne $lastar) {
	    $i = 0;
	} elsif ($album ne $lastal) {
	    $i = 1;
	}
#print "to level ", $i, "\n";
	$lastar = $artist;
	$lastal = $album;
	while ($indent > $i) {
	    if ($indent == 2) {
#		print {$html{$cb}} "</table></td><td align=center>X</td></tr></table>\n"
		if ($albumicon ne "") {
		    print {$html{$cb}} "</table></td><td align=center><img src=\"$albumicon\"></td></tr></table>\n"
		} else {
		    print {$html{$cb}} "</table></td><td align=center></td></tr></table>\n"
		}
	    }
	    if ($indent == 1) {
		print {$html{$cb}} "</ul>\n"
	    }
	    $indent--;
	}
	$i = 2;
	while ($indent < $i) {
	    $t = 1;
	    $indent++;
	    if ($indent == 1) {
		while ($lastalf lt btag($sortk)) {
		    $lastalf = chr(ord($lastalf) + 1);
		    print {$html{$cb}} "<a name=\"", $lastalf, "\">\n";
		}
		print {$html{$cb}} "<a name=\"", $artist, "\">\n";
		print {$html{$cb}} "<table width=50%>\n";
		print {$html{$cb}} "<tr><td align=left><p><font size=+2>", $artist, "</font></td>\n";
		print {$html{$cb}} "<td align=right>\n";
		print {$html{$cb}} "<table>\n";
		print {$html{$cb}} "<tr><td colspan=3 align=center><a href=\"#TOPOFILE\">\n";
		print {$html{$cb}} "<img src=\"/rfm/upto.gif\" alt=\"to Top\" border=0></a></td></tr>\n";
		print {$html{$cb}} "<tr><td align=center>";
		if ($prevcb ne "") {
		    print {$html{$cb}} "<a href=\"tunesbyartist", $prevcb, ".html\"><img src=\"/rfm/upup.gif\" alt=\"browse Back\" border=0></a>\n";
		}
		print {$html{$cb}} "</td>\n";
		print {$html{$cb}} "<td align=center><a href=\"/rfm/artists.html\"><img src=\"/rfm/ana.gif\" alt=\"to Index\" border=0></a></td>\n";
		print {$html{$cb}} "<td align=center>";
		if ($nextcb ne "") {
		    print {$html{$cb}} "<a href=\"tunesbyartist", $nextcb, ".html\"><img src=\"/rfm/downdown.gif\" alt=\"browse Fwd\" border=0></a>\n";
		}
		print {$html{$cb}} "</td></tr>\n";
		print {$html{$cb}} "<tr><td colspan=3 align=center><a href=\"#ENDOFILE\">\n";
		print {$html{$cb}} "<img src=\"/rfm/downto.gif\" alt=\"to Bottom\" border=0></a></td></tr>\n";
		print {$html{$cb}} "</table>\n";
		print {$html{$cb}} "</td>\n";
		print {$html{$cb}} "</tr>\n";
		print {$html{$cb}} "</table>\n";
		print {$html{$cb}} "<ul>\n";
	    } elsif ($indent == 2) {
		print AUTOTAGS $albumfile . ":A_ALBUM=" . $artist . "/" . $album . "\n";
		print {$html{$cb}} "<a name=\"", $artist, "/", $album, "\">\n";
		print {$html{$cb}} "<li><b> $album </b>\n";
		print {$html{$cb}} "<a href=\"/cgi-bin/tuneput.cgi?album=" . $albumfile . "\"><img border=0 align=middle alt=\"Play\" src=\"play.gif\"></a>\n";
		print {$html{$cb}} "<a href=\"/cgi-bin/playalbum.cgi?album=" . $albumfile . "\"><img border=0 align=middle alt=\"Play Selected Tracks\" src=\"playoption.gif\"></a>\n";
		if ($tags{$albumfile, "DATE"} ne "") {
		    print {$html{$cb}} "<br><em>" . $tags{$albumfile, "DATE"} . "</em>\n";
		}
#		print {$html{$cb}} "<table width=50% cellpadding=0>\n";
		print {$html{$cb}} "<table width=90% cellpadding=0>\n";
		print {$html{$cb}} "<tr><td width=60%>\n";
		print {$html{$cb}} "<table width=100% cellpadding=0>\n";
if ($tags{$albumfile, "ICON"} ne "") {
    $albumicon=$albumfile . "/" . $tags{$albumfile, "ICON"};
    $albumicon =~ s/^\///;
    $albumicon =~ s/\//./g;
    $albumicon="/rfm/icons/". $albumicon;
} else {
    $albumicon="";
}
	    }
	}
	$ttime = $tags{$path, "T"};
	print {$html{$cb}} "<tr>";
	print {$html{$cb}} "<td align=right><font size=-1>$t.</font> <a href=\"/cgi-bin/tuneput.cgi?";
	$t++;
	print {$html{$cb}} $canons{$sortk};
	print {$html{$cb}} "\">";
	print {$html{$cb}} $track;
	print {$html{$cb}} "</a></td>";
	print {$html{$cb}} "<td width=25% align=center><a href=\"/cgi-bin/abouttrack.cgi?";
	print {$html{$cb}} $canons{$sortk};
	print {$html{$cb}} "\">", $ttime, "</a></td>\n";
	print {$html{$cb}} "</tr>\n";
    }
    while ($indent > 0) {
	if ($indent == 2) {
	    if ($albumicon ne "") {
		print {$html{$cb}} "</table></td><td align=center><img src=\"$albumicon\"></td></tr></table>\n"
	    } else {
		print {$html{$cb}} "</table></td><td align=center></td></tr></table>\n"
	    }
	}
	if ($indent == 1) {
	    print {$html{$cb}} "</ul>\n"
	}
	$indent--;
    }

    print {$html{$cb}} "<table width=50%>\n";
    print {$html{$cb}} "<tr><td></td>\n";
    print {$html{$cb}} "<td align=right>\n";
    print {$html{$cb}} "<table>\n";
    print {$html{$cb}} "<tr><td colspan=3 align=center><a href=\"#TOPOFILE\">\n";
    print {$html{$cb}} "<img src=\"/rfm/upto.gif\" alt=\"to Top\" border=0></a></td></tr>\n";
    print {$html{$cb}} "<tr><td align=center>";
    if ($prevcb ne "") {
	print {$html{$cb}} "<a href=\"tunesbyartist", $prevcb, ".html\"><img src=\"/rfm/upup.gif\" alt=\"browse Back\" border=0></a>\n";
    }
    print {$html{$cb}} "</td>\n";
    print {$html{$cb}} "<td align=center><a href=\"/rfm/artists.html\"><img src=\"/rfm/ana.gif\" alt=\"to Index\" border=0></a></td>\n";
    print {$html{$cb}} "<td align=center>";
    if ($nextcb ne "") {
	print {$html{$cb}} "<a href=\"tunesbyartist", $nextcb, ".html\"><img src=\"/rfm/downdown.gif\" alt=\"browse Fwd\" border=0></a>\n";
    }
    print {$html{$cb}} "</td></tr>\n";
    print {$html{$cb}} "<tr><td colspan=3 align=center><a href=\"#ENDOFILE\">\n";
    print {$html{$cb}} "<img src=\"/rfm/downto.gif\" alt=\"to Bottom\" border=0></a></td></tr>\n";
    print {$html{$cb}} "</table>\n";
    print {$html{$cb}} "</td>\n";
    print {$html{$cb}} "</tr>\n";
    print {$html{$cb}} "</table>\n";

    print {$html{$cb}} "<p><center><b>\n";
    $i = "";
    foreach $c (sort(keys(coalbucket))) {
	print {$html{$cb}} $i, "<a href=\"tunesbyartist", $coalbucket{$c}, ".html#", $c, "\">", $c, "</a>\n";
	$i = "-";
    }
    print {$html{$cb}} "</b><br>\n";
    print {$html{$cb}} "</center><p>\n";
    print {$html{$cb}} "<center>click to make a request\n";
    print {$html{$cb}} "<br><font size=+2>Tunes by Artist ", $c1, "-", $c2, "</font></center>\n";

    open(I, $ENV{TUNEBOT_ROOT} . "/html/bodypart.html.tail");
    while ($line = <I>) {
	print {$html{$cb}} $line;
    }
    close(I);
    print {$html{$cb}} "</html>\n";
    close($html{$cb});
}

#
# grind out the overall artists index.
#
print " artists.html\n";
open(ARTISTS, ">" . $ENV{TUNEBOT_ROOT} . "/html/artists.html");
print ARTISTS "<html>\n";
print ARTISTS "<head>\n";
print ARTISTS "<META HTTP-EQUIV=\"Pragma\" CONTENT=\"no-cache\">\n";
$c1 = substr($cb, 0, 1);
$c2 = substr($cb, length($cb) - 1);
print ARTISTS "<title>" . $ENV{BANNER_STRING} . " Fast Index of Artists</title>\n";
print ARTISTS "</head>\n";
open(I, $ENV{TUNEBOT_ROOT} . "/html/bodypart.html.head");
while ($line = <I>) {
    print ARTISTS $line;
}
close(I);
print ARTISTS "<center><font size=+2>Fast Index of Artists</font>\n";
print ARTISTS "<br>click to see albums</center>\n";
print ARTISTS "<p><center><b>\n";
$i = "";
foreach $c (sort(keys(coalbucket))) {
    print ARTISTS $i, "<a href=\"tunesbyartist", $coalbucket{$c}, ".html#", $c, "\">", $c, "</a>\n";
    $i = "-";
}
print ARTISTS "</b></center><p>\n";
@sarts = keys(artists);
foreach $k (@sarts) {
    $alfes{btag($k)} = 1;
}
$narts = $#sarts + 1;
@sarts = sort folded (@sarts, keys(alfes));
print ARTISTS "<center><font size=-1><b> - ", $narts, " Artists - ", $ntunes, " Tracks - </b></font></center><p>\n";
$narts = $#sarts + 1;
$nrows = int(($narts + 2) / 3);
print ARTISTS "<ul>\n";
print ARTISTS "<table width=90%>\n";
for ($i = 0; $i < $nrows; $i++) {
    print ARTISTS "<tr>\n";
    for ($j = 0; $j < $nrows * 3; $j += $nrows) {
	$k = $j + $i;
	if ($k < $narts) {
	    if ($alfes{$sarts[$k]} == 1) {
		$cb = $coalbucket{btag($sarts[$k])};
		print ARTISTS "<td width=33%><b><font size=+1>- </font>";
		print ARTISTS "<a href=\"tunesbyartist", $cb, ".html#", $sarts[$k], "\">";
		print ARTISTS "<font size=+1 color=black><b>";
		print ARTISTS $sarts[$k];
		print ARTISTS "</b></font></a><font size=+1> -</font></b></td>";
	    } else {
		$a = $artists{$sarts[$k]};
		$aa = $a;
		if ($boldartists{$sarts[$k]} != 0) {
		    $aa = "<font size=+1><b>" . $a . "</b></font>";
		}
		$cb = $coalbucket{btag($sarts[$k])};
		print ARTISTS "<td width=33%><a href=\"/rfm/tunesbyartist", $cb, ".html#", $a, "\">", $aa, "</a></td>\n";
	    }
	}
    }
    print ARTISTS "</tr>\n";
}
print ARTISTS "</table>\n";
print ARTISTS "</ul>\n";
print ARTISTS "<p><center><b>\n";
$i = "";
foreach $c (sort(keys(coalbucket))) {
    print ARTISTS $i, "<a href=\"tunesbyartist", $coalbucket{$c}, ".html#", $c, "\">", $c, "</a>\n";
    $i = "-";
}
print ARTISTS "</b></center><p>\n";
open(I, $ENV{TUNEBOT_ROOT} . "/html/bodypart.html.tail");
while ($line = <I>) {
    print ARTISTS $line;
}
close(I);
print ARTISTS "</html>\n";
close(ARTISTS);

print " tunes-bytitle.html\n";
open(BYTITLE, ">" . $ENV{TUNEBOT_ROOT} . "/html/tunes-bytitle.html");
open(I, $ENV{TUNEBOT_ROOT} . "/html/tunes-bytitle.html.head");
while ($line = <I>) {
    print BYTITLE $line;
}
close(I);
open(I, $ENV{TUNEBOT_ROOT} . "/html/bodypart.html.head");
while ($line = <I>) {
    print BYTITLE $line;
}
close(I);
open(I, $ENV{TUNEBOT_ROOT} . "/html/tunes-bytitle.html.body");
while ($line = <I>) {
    print BYTITLE $line;
}
close(I);

print BYTITLE "<p><center><b>\n";
$i = "";
foreach $c (A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,Q,R,S,T,U,V,W,X,Y,Z) {
    print BYTITLE $i, "<a href=\"tunes-bytitle.html#", $c, "\">", $c, "</a>\n";
    $i = "-";
}
print BYTITLE "</b></center><p>\n";
print BYTITLE "<p><center><b><font size=-1>\n";
print BYTITLE "- " . $ntunes . " Tracks -\n";
print BYTITLE "</font></b></center>\n";
    @sortks = ();
    open(TUNEIX, $ENV{TUNEIX});
    $i = 0;
    while ($path = <TUNEIX>) {
	chop $path;
	$canon = $path;
#	$i = index($path, "\.mp3");
#	if ($i >= 0) {
#	    $path = substr($path, 0, $i);
#	}
	$path =~ s/\.mp3$//;
	@raw = split('/', $path);
	$n = $#raw;
	$artistfile = join("/", @raw[0..$n-2]);
	$artist = $tags{$artistfile, "N"};
	if ($artist eq "") {
	    $artist = $raw[$n-2];
	}
	$sartist = $tags{$artistfile, "SORT"};
	if ($sartist eq "") {
	    $sartist = $artist;
	}
	$albumfile = join("/", @raw[0..$n-1]);
	$album = $tags{$albumfile, "N"};
	if ($album eq "") {
	    $album = $raw[$n-1];
	}
	$track = $tags{$path, "N"};
	if ($track eq "") {
	    $track = $raw[$n];
	}
	$sortk = $track . $canon;
#	print $canon, "\n";
#	print " SORTS ON ", $sortk, "\n";
#	print "  RENDERS AS ", $artist . "/" . $album . "/" . $track, "\n";
#	print "   GOES INTO ", $coalbucket{btag($sartist)}, "\n";
#	@sortks = (@sortks, $sortk);
	$sortks[$i++] = $sortk;
	$canons{$sortk} = $canon;
#	$render{$sortk} = join("/", $artist, $album, $track);
    }
    close(TUNEIX);
    @sortks = sort folded @sortks;
    $lastalf = "A";
    $lastalf = chr(ord($lastalf) - 1);
#    print BYTITLE "<table width=90% align=center>\n";
    $linenumber = 0;
    foreach $sortk (@sortks) {
	$linenumber++;
	$canon = $canons{$sortk};
	$path = $canon;
	$i = index($path, "\.mp3");
	if ($i >= 0) {
	    $path = substr($path, 0, $i);
	}
	@raw = split('/', $path);
	$n = $#raw;
	$artistfile = join("/", @raw[0..$n-2]);
	$artist = $tags{$artistfile, "N"};
	if ($artist eq "") {
	    $artist = $raw[$n-2];
	}
	$sartist = $tags{$artistfile, "SORT"};
	if ($sartist eq "") {
	    $sartist = $artist;
	}
	$albumfile = join("/", @raw[0..$n-1]);
	$album = $tags{$albumfile, "N"};
	if ($album eq "") {
	    $album = $raw[$n-1];
	}
	$track = $tags{$path, "N"};
	if ($track eq "") {
	    $track = $raw[$n];
	}
	if ($tags{$path, "ARTIST"} ne "") {
	    $track = $track . "<em> - " . $tags{$path, "ARTIST"} . "</em>";
	}
	$i = 0;
	while ($lastalf lt btag($sortk)) {
	    $lastalf = chr(ord($lastalf) + 1);
	    print BYTITLE "<a name=\"", $lastalf, "\">\n";
	    $i = 1;
	}
	if ($i) {
	    print BYTITLE "<p><font size=+1><b>- ", $lastalf, " -</b></font><p>\n";
	    $linenumber = 1;
	}
	$ttime = $tags{$path, "T"};
	print BYTITLE "<table width=90% align=center cellspacing=0>\n";
	print BYTITLE "<tr>";
	print BYTITLE "<td width=40%><a href=\"/cgi-bin/tuneput.cgi?" . $canon . "\">" . $track . "</a></td>";
	print BYTITLE "<td width=10%><a href=\"/cgi-bin/abouttrack.cgi?";
	print BYTITLE $canons{$sortk} . "\">" . $ttime . "</a></td>";
	print BYTITLE "<td>\n";
	if ($linenumber % 10 == 1) {
	    print BYTITLE "<a href=\"#TOPOFILE\"><img src=\"/rfm/upto.gif\" alt=\"to Top\" border=0></a>\n";
	}
	print BYTITLE "</td>\n";
	print BYTITLE "<td width=40%>";
	print BYTITLE "<a href=\"/rfm/tunesbyartist" . $coalbucket{btag($sartist)} . ".html#" . $artist . "\">" . $artist . "</a> / ";
	print BYTITLE "<a href=\"/rfm/tunesbyartist" . $coalbucket{btag($sartist)} . ".html#" . $artist . "/" . $album . "\">" . $album . "</a></td>\n";
	print BYTITLE "</tr>\n";
	print BYTITLE "</table>\n";
    }
#    print BYTITLE "</table>\n";

open(I, $ENV{TUNEBOT_ROOT} . "/html/bodypart.html.tail");
while ($line = <I>) {
    print BYTITLE $line;
}
close(I);
open(I, $ENV{TUNEBOT_ROOT} . "/html/tunes-bytitle.html.tail");
while ($line = <I>) {
    print BYTITLE $line;
}
close(I);
close(BYTITLE);

close(AUTOTAGS);

